@@ -27,9 +27,9 @@ Z_transform : Transform
     transforms Z_lmn coefficients to real space
 L_transform : Transform
     transforms L_lmn coefficients to real space
-p_transform : Transform
+p_profile : Profile
     transforms p_l coefficients to real space
-i_transform : Transform
+i_profile : Profile
     transforms i_l coefficients to real space
 
 """
@@ -87,8 +87,8 @@ def compute_profiles(
     R_transform,
     Z_transform,
     L_transform,
-    p_transform,
-    i_transform,
+    p_profile,
+    i_profile,
 ):
     """Compute magnetic flux, pressure, and rotational transform profiles.
 
@@ -112,9 +112,9 @@ def compute_profiles(
         transforms Z_lmn coefficients to real space
     L_transform : Transform
         transforms L_lmn coefficients to real space
-    p_transform : Transform
+    p_profile : Profile
         transforms p_l coefficients to real space
-    i_transform : Transform
+    i_profile : Profile
         transforms i_l coefficients to real space
 
     Returns
@@ -126,19 +126,19 @@ def compute_profiles(
     """
     profiles = {}
 
-    # toroidal flux
-    rho = p_transform.grid.nodes[:, 0]
-    profiles["psi"] = Psi * rho ** 2
-    profiles["psi_r"] = 2 * Psi * rho
-    profiles["psi_rr"] = 2 * Psi * np.ones_like(rho)
+    # toroidal flux (Wb) divided by 2 pi
+    rho = p_profile.grid.nodes[:, 0]
+    profiles["psi"] = Psi * rho ** 2 / (2 * jnp.pi)
+    profiles["psi_r"] = 2 * Psi * rho / (2 * jnp.pi)
+    profiles["psi_rr"] = 2 * Psi * np.ones_like(rho) / (2 * jnp.pi)
 
-    # pressure
-    profiles["p"] = p_transform.transform(p_l, 0)
-    profiles["p_r"] = p_transform.transform(p_l, 1)
+    # pressure (Pa)
+    profiles["p"] = p_profile.compute(p_l, dr=0)
+    profiles["p_r"] = p_profile.compute(p_l, dr=1)
 
     # rotational transform
-    profiles["iota"] = i_transform.transform(i_l, 0)
-    profiles["iota_r"] = i_transform.transform(i_l, 1)
+    profiles["iota"] = i_profile.compute(i_l, dr=0)
+    profiles["iota_r"] = i_profile.compute(i_l, dr=1)
 
     return profiles
 
@@ -153,8 +153,8 @@ def compute_toroidal_coords(
     R_transform,
     Z_transform,
     L_transform,
-    p_transform,
-    i_transform,
+    p_profile,
+    i_profile,
 ):
     """Transform toroidal coordinates to real space.
 
@@ -178,9 +178,9 @@ def compute_toroidal_coords(
         transforms Z_lmn coefficients to real space
     L_transform : Transform
         transforms L_lmn coefficients to real space
-    p_transform : Transform
+    p_profile : Profile
         transforms p_l coefficients to real space
-    i_transform : Transform
+    i_profile : Profile
         transforms i_l coefficients to real space
 
     Returns
@@ -209,8 +209,8 @@ def compute_cartesian_coords(
     R_transform,
     Z_transform,
     L_transform,
-    p_transform,
-    i_transform,
+    p_profile,
+    i_profile,
 ):
     """Compute cartesian coordinates from toroidal coordinates.
 
@@ -234,9 +234,9 @@ def compute_cartesian_coords(
         transforms Z_lmn coefficients to real space
     L_transform : Transform
         transforms L_lmn coefficients to real space
-    p_transform : Transform
+    p_profile : Profile
         transforms p_l coefficients to real space
-    i_transform : Transform
+    i_profile : Profile
         transforms i_l coefficients to real space
 
     Returns
@@ -261,8 +261,8 @@ def compute_cartesian_coords(
         R_transform,
         Z_transform,
         L_transform,
-        p_transform,
-        i_transform,
+        p_profile,
+        i_profile,
     )
 
     cartesian_coords = {}
@@ -284,8 +284,8 @@ def compute_covariant_basis(
     R_transform,
     Z_transform,
     L_transform,
-    p_transform,
-    i_transform,
+    p_profile,
+    i_profile,
 ):
     """Compute covariant basis vectors.
 
@@ -309,9 +309,9 @@ def compute_covariant_basis(
         transforms Z_lmn coefficients to real space
     L_transform : Transform
         transforms L_lmn coefficients to real space
-    p_transform : Transform
+    p_profile : Profile
         transforms p_l coefficients to real space
-    i_transform : Transform
+    i_profile : Profile
         transforms i_l coefficients to real space
 
     Returns
@@ -336,8 +336,8 @@ def compute_covariant_basis(
         R_transform,
         Z_transform,
         L_transform,
-        p_transform,
-        i_transform,
+        p_profile,
+        i_profile,
     )
 
     # toroidal coordinate 1st derivatives
@@ -372,8 +372,8 @@ def compute_jacobian(
     R_transform,
     Z_transform,
     L_transform,
-    p_transform,
-    i_transform,
+    p_profile,
+    i_profile,
 ):
     """Compute coordinate system jacobian.
 
@@ -397,9 +397,9 @@ def compute_jacobian(
         transforms Z_lmn coefficients to real space
     L_transform : Transform
         transforms L_lmn coefficients to real space
-    p_transform : Transform
+    p_profile : Profile
         transforms p_l coefficients to real space
-    i_transform : Transform
+    i_profile : Profile
         transforms i_l coefficients to real space
 
     Returns
@@ -428,8 +428,8 @@ def compute_jacobian(
         R_transform,
         Z_transform,
         L_transform,
-        p_transform,
-        i_transform,
+        p_profile,
+        i_profile,
     )
 
     jacobian = {}
@@ -450,8 +450,8 @@ def compute_contravariant_basis(
     R_transform,
     Z_transform,
     L_transform,
-    p_transform,
-    i_transform,
+    p_profile,
+    i_profile,
 ):
     """Compute contravariant basis vectors.
 
@@ -475,9 +475,9 @@ def compute_contravariant_basis(
         transforms Z_lmn coefficients to real space
     L_transform : Transform
         transforms L_lmn coefficients to real space
-    p_transform : Transform
+    p_profile : Profile
         transforms p_l coefficients to real space
-    i_transform : Transform
+    i_profile : Profile
         transforms i_l coefficients to real space
 
     Returns
@@ -510,8 +510,8 @@ def compute_contravariant_basis(
         R_transform,
         Z_transform,
         L_transform,
-        p_transform,
-        i_transform,
+        p_profile,
+        i_profile,
     )
 
     con_basis = {}
@@ -538,8 +538,8 @@ def compute_magnetic_field(
     R_transform,
     Z_transform,
     L_transform,
-    p_transform,
-    i_transform,
+    p_profile,
+    i_profile,
 ):
     """Compute magnetic field components.
 
@@ -563,9 +563,9 @@ def compute_magnetic_field(
         transforms Z_lmn coefficients to real space
     L_transform : Transform
         transforms L_lmn coefficients to real space
-    p_transform : Transform
+    p_profile : Profile
         transforms p_l coefficients to real space
-    i_transform : Transform
+    i_profile : Profile
         transforms i_l coefficients to real space
 
     Returns
@@ -602,8 +602,8 @@ def compute_magnetic_field(
         R_transform,
         Z_transform,
         L_transform,
-        p_transform,
-        i_transform,
+        p_profile,
+        i_profile,
     )
     jacobian, cov_basis, toroidal_coords = compute_jacobian(
         Psi,
@@ -615,8 +615,8 @@ def compute_magnetic_field(
         R_transform,
         Z_transform,
         L_transform,
-        p_transform,
-        i_transform,
+        p_profile,
+        i_profile,
     )
 
     # lambda derivatives
@@ -624,7 +624,7 @@ def compute_magnetic_field(
     toroidal_coords["lambda_z"] = L_transform.transform(L_lmn, 0, 0, 1)
 
     magnetic_field = {}
-    magnetic_field["B0"] = profiles["psi_r"] / (2 * jnp.pi * jacobian["g"])
+    magnetic_field["B0"] = profiles["psi_r"] / jacobian["g"]
 
     # contravariant components
     magnetic_field["B^rho"] = jnp.zeros_like(magnetic_field["B0"])
@@ -642,6 +642,11 @@ def compute_magnetic_field(
     magnetic_field["B_theta"] = dot(magnetic_field["B"], cov_basis["e_theta"], 0)
     magnetic_field["B_zeta"] = dot(magnetic_field["B"], cov_basis["e_zeta"], 0)
 
+    # cylindrical components (R,Z,phi)
+    magnetic_field["B_R"] = magnetic_field['B^theta']*cov_basis['e_theta'][0,:] + magnetic_field['B^zeta']*cov_basis['e_zeta'][0,:]
+    magnetic_field["B_Z"] = magnetic_field['B^theta']*cov_basis['e_theta'][2,:] + magnetic_field['B^zeta']*cov_basis['e_zeta'][2,:]
+    magnetic_field["B_phi"] = toroidal_coords["R"]*magnetic_field['B^zeta']
+
     return magnetic_field, jacobian, cov_basis, toroidal_coords, profiles
 
 
@@ -655,8 +660,8 @@ def compute_magnetic_field_axis(
     R_transform,
     Z_transform,
     L_transform,
-    p_transform,
-    i_transform,
+    p_profile,
+    i_profile,
 ):
     """Compute magnetic field components; can handle nodes at the magnetic axis.
 
@@ -680,9 +685,9 @@ def compute_magnetic_field_axis(
         transforms Z_lmn coefficients to real space
     L_transform : Transform
         transforms L_lmn coefficients to real space
-    p_transform : Transform
+    p_profile : Profile
         transforms p_l coefficients to real space
-    i_transform : Transform
+    i_profile : Profile
         transforms i_l coefficients to real space
 
     Returns
@@ -719,8 +724,8 @@ def compute_magnetic_field_axis(
         R_transform,
         Z_transform,
         L_transform,
-        p_transform,
-        i_transform,
+        p_profile,
+        i_profile,
     )
     jacobian, cov_basis, toroidal_coords = compute_jacobian(
         Psi,
@@ -732,11 +737,11 @@ def compute_magnetic_field_axis(
         R_transform,
         Z_transform,
         L_transform,
-        p_transform,
-        i_transform,
+        p_profile,
+        i_profile,
     )
 
-    axis = i_transform.grid.axis
+    axis = i_profile.grid.axis
 
     # lambda derivatives
     toroidal_coords["lambda_t"] = L_transform.transform(L_lmn, 0, 1, 0)
@@ -775,11 +780,9 @@ def compute_magnetic_field_axis(
     )
 
     magnetic_field = {}
-    magnetic_field["B0"] = profiles["psi_r"] / (2 * jnp.pi * jacobian["g"])
+    magnetic_field["B0"] = profiles["psi_r"] / jacobian["g"]
     magnetic_field["B0"] = put(
-        magnetic_field["B0"],
-        axis,
-        profiles["psi_rr"][axis] / (2 * jnp.pi * jacobian["g_r"][axis]),
+        magnetic_field["B0"], axis, profiles["psi_rr"][axis] / jacobian["g_r"][axis],
     )
 
     # contravariant components
@@ -798,6 +801,11 @@ def compute_magnetic_field_axis(
     magnetic_field["B_theta"] = dot(magnetic_field["B"], cov_basis["e_theta"], 0)
     magnetic_field["B_zeta"] = dot(magnetic_field["B"], cov_basis["e_zeta"], 0)
 
+    # cylindrical components (R,Z,phi)
+    magnetic_field["B_R"] = magnetic_field['B^theta']*cov_basis['e_theta'][0,:] + magnetic_field['B^zeta']*cov_basis['e_zeta'][0,:]
+    magnetic_field["B_Z"] = magnetic_field['B^theta']*cov_basis['e_theta'][2,:] + magnetic_field['B^zeta']*cov_basis['e_zeta'][2,:]
+    magnetic_field["B_phi"] = toroidal_coords["R"]*magnetic_field['B^zeta']
+
     return magnetic_field, jacobian, cov_basis, toroidal_coords, profiles
 
 
@@ -811,8 +819,8 @@ def compute_magnetic_field_magnitude(
     R_transform,
     Z_transform,
     L_transform,
-    p_transform,
-    i_transform,
+    p_profile,
+    i_profile,
 ):
     """Compute magnetic field magnitude.
 
@@ -836,9 +844,9 @@ def compute_magnetic_field_magnitude(
         transforms Z_lmn coefficients to real space
     L_transform : Transform
         transforms L_lmn coefficients to real space
-    p_transform : Transform
+    p_profile : Profile
         transforms p_l coefficients to real space
-    i_transform : Transform
+    i_profile : Profile
         transforms i_l coefficients to real space
 
     Returns
@@ -881,8 +889,8 @@ def compute_magnetic_field_magnitude(
         R_transform,
         Z_transform,
         L_transform,
-        p_transform,
-        i_transform,
+        p_profile,
+        i_profile,
     )
 
     magnetic_field["|B|"] = jnp.sqrt(
@@ -909,8 +917,8 @@ def compute_magnetic_field_magnitude_axis(
     R_transform,
     Z_transform,
     L_transform,
-    p_transform,
-    i_transform,
+    p_profile,
+    i_profile,
 ):
     """Compute magnetic field magnitude; can handle nodes at the magnetic axis.
 
@@ -934,9 +942,9 @@ def compute_magnetic_field_magnitude_axis(
         transforms Z_lmn coefficients to real space
     L_transform : Transform
         transforms L_lmn coefficients to real space
-    p_transform : Transform
+    p_profile : Transform
         transforms p_l coefficients to real space
-    i_transform : Transform
+    i_profile : Profile
         transforms i_l coefficients to real space
 
     Returns
@@ -979,8 +987,8 @@ def compute_magnetic_field_magnitude_axis(
         R_transform,
         Z_transform,
         L_transform,
-        p_transform,
-        i_transform,
+        p_profile,
+        i_profile,
     )
 
     magnetic_field["|B|"] = jnp.sqrt(
@@ -1007,8 +1015,8 @@ def compute_current_density(
     R_transform,
     Z_transform,
     L_transform,
-    p_transform,
-    i_transform,
+    p_profile,
+    i_profile,
 ):
     """Compute current density field components.
 
@@ -1032,9 +1040,9 @@ def compute_current_density(
         transforms Z_lmn coefficients to real space
     L_transform : Transform
         transforms L_lmn coefficients to real space
-    p_transform : Transform
+    p_profile : Profile
         transforms p_l coefficients to real space
-    i_transform : Transform
+    i_profile : Profile
         transforms i_l coefficients to real space
 
     Returns
@@ -1081,8 +1089,8 @@ def compute_current_density(
         R_transform,
         Z_transform,
         L_transform,
-        p_transform,
-        i_transform,
+        p_profile,
+        i_profile,
     )
 
     # toroidal coordinate 2nd derivatives
@@ -1173,15 +1181,12 @@ def compute_current_density(
     )
 
     # B contravariant component derivatives
-    magnetic_field["B0_r"] = profiles["psi_rr"] / (2 * jnp.pi * jacobian["g"]) - (
-        profiles["psi_r"] * jacobian["g_r"]
-    ) / (2 * jnp.pi * jacobian["g"] ** 2)
-    magnetic_field["B0_t"] = -(profiles["psi_r"] * jacobian["g_t"]) / (
-        2 * jnp.pi * jacobian["g"] ** 2
-    )
-    magnetic_field["B0_z"] = -(profiles["psi_r"] * jacobian["g_z"]) / (
-        2 * jnp.pi * jacobian["g"] ** 2
+    magnetic_field["B0_r"] = (
+        profiles["psi_rr"] / jacobian["g"]
+        - (profiles["psi_r"] * jacobian["g_r"]) / jacobian["g"] ** 2
     )
+    magnetic_field["B0_t"] = -(profiles["psi_r"] * jacobian["g_t"]) / jacobian["g"] ** 2
+    magnetic_field["B0_z"] = -(profiles["psi_r"] * jacobian["g_z"]) / jacobian["g"] ** 2
     magnetic_field["B^theta_r"] = magnetic_field["B0_r"] * (
         profiles["iota"] - toroidal_coords["lambda_z"]
     ) + magnetic_field["B0"] * (profiles["iota_r"] - toroidal_coords["lambda_rz"])
@@ -1279,8 +1284,8 @@ def compute_magnetic_pressure_gradient(
     R_transform,
     Z_transform,
     L_transform,
-    p_transform,
-    i_transform,
+    p_profile,
+    i_profile,
 ):
     """Compute magnetic pressure gradient components and its magnitude.
 
@@ -1304,9 +1309,9 @@ def compute_magnetic_pressure_gradient(
         transforms Z_lmn coefficients to real space
     L_transform : Transform
         transforms L_lmn coefficients to real space
-    p_transform : Transform
+    p_profile : Profile
         transforms p_l coefficients to real space
-    i_transform : Transform
+    i_profile : Profile
         transforms i_l coefficients to real space
 
     Returns
@@ -1362,8 +1367,8 @@ def compute_magnetic_pressure_gradient(
         R_transform,
         Z_transform,
         L_transform,
-        p_transform,
-        i_transform,
+        p_profile,
+        i_profile,
     )
 
     # contravariant basis vectors
@@ -1450,8 +1455,8 @@ def compute_magnetic_tension(
     R_transform,
     Z_transform,
     L_transform,
-    p_transform,
-    i_transform,
+    p_profile,
+    i_profile,
 ):
     """Compute magnetic tension vector and its magnitude.
 
@@ -1475,9 +1480,9 @@ def compute_magnetic_tension(
         transforms Z_lmn coefficients to real space
     L_transform : Transform
         transforms L_lmn coefficients to real space
-    p_transform : Transform
+    p_profile : Profile
         transforms p_l coefficients to real space
-    i_transform : Transform
+    i_profile : Profile
         transforms i_l coefficients to real space
 
     Returns
@@ -1533,8 +1538,8 @@ def compute_magnetic_tension(
         R_transform,
         Z_transform,
         L_transform,
-        p_transform,
-        i_transform,
+        p_profile,
+        i_profile,
     )
 
     # contravariant basis vectors
@@ -1616,8 +1621,8 @@ def compute_force_error(
     R_transform,
     Z_transform,
     L_transform,
-    p_transform,
-    i_transform,
+    p_profile,
+    i_profile,
 ):
     """Compute force error components.
 
@@ -1641,9 +1646,9 @@ def compute_force_error(
         transforms Z_lmn coefficients to real space
     L_transform : Transform
         transforms L_lmn coefficients to real space
-    p_transform : Transform
+    p_profile : Profile
         transforms p_l coefficients to real space
-    i_transform : Transform
+    i_profile : Profile
         transforms i_l coefficients to real space
 
     Returns
@@ -1695,8 +1700,8 @@ def compute_force_error(
         R_transform,
         Z_transform,
         L_transform,
-        p_transform,
-        i_transform,
+        p_profile,
+        i_profile,
     )
 
     force_error = {}
@@ -1731,8 +1736,8 @@ def compute_force_error_magnitude(
     R_transform,
     Z_transform,
     L_transform,
-    p_transform,
-    i_transform,
+    p_profile,
+    i_profile,
 ):
     """Compute force error magnitude.
 
@@ -1756,9 +1761,9 @@ def compute_force_error_magnitude(
         transforms Z_lmn coefficients to real space
     L_transform : Transform
         transforms L_lmn coefficients to real space
-    p_transform : Transform
+    p_profile : Profile
         transforms p_l coefficients to real space
-    i_transform : Transform
+    i_profile : Profile
         transforms i_l coefficients to real space
 
     Returns
@@ -1815,8 +1820,8 @@ def compute_force_error_magnitude(
         R_transform,
         Z_transform,
         L_transform,
-        p_transform,
-        i_transform,
+        p_profile,
+        i_profile,
     )
 
     # contravariant basis vectors
@@ -1877,11 +1882,12 @@ def compute_energy(
     R_transform,
     Z_transform,
     L_transform,
-    p_transform,
-    i_transform,
+    p_profile,
+    i_profile,
 ):
     """Compute MHD energy by quadrature sum.
-
+    :math:`\int_V dV(\\frac{B^2}{2\mu_0} + \\frac{p}{\gamma - 1})`
+    where DESC assumes :math:`\gamma=0`
     **REQUIRES 'quad' grid for correct results**
 
     Parameters
@@ -1904,9 +1910,9 @@ def compute_energy(
         transforms Z_lmn coefficients to real space
     L_transform : Transform
         transforms L_lmn coefficients to real space
-    p_transform : Transform
+    p_profile : Profile
         transforms p_l coefficients to real space
-    i_transform : Transform
+    i_profile : Profile
         transforms i_l coefficients to real space
 
     Returns
@@ -1951,8 +1957,8 @@ def compute_energy(
         R_transform,
         Z_transform,
         L_transform,
-        p_transform,
-        i_transform,
+        p_profile,
+        i_profile,
     )
 
     NFP = R_transform.grid.NFP
@@ -1965,7 +1971,7 @@ def compute_energy(
         / (2 * mu_0)
         * NFP
     )
-    energy["W"] = energy["W_B"] - energy["W_p"]
+    energy["W"] = energy["W_B"] + energy["W_p"]
     energy["beta"] = jnp.abs(energy["W_p"] / energy["W_B"])
 
     return (energy, magnetic_field, jacobian, cov_basis, toroidal_coords, profiles)
@@ -1981,8 +1987,8 @@ def compute_quasisymmetry(
     R_transform,
     Z_transform,
     L_transform,
-    p_transform,
-    i_transform,
+    p_profile,
+    i_profile,
 ):
     """Compute quasisymmetry metrics.
 
@@ -2006,9 +2012,9 @@ def compute_quasisymmetry(
         transforms Z_lmn coefficients to real space
     L_transform : Transform
         transforms L_lmn coefficients to real space
-    p_transform : Transform
+    p_profile : Profile
         transforms p_l coefficients to real space
-    i_transform : Transform
+    i_profile : Profile
         transforms i_l coefficients to real space
 
     Returns
@@ -2060,11 +2066,11 @@ def compute_quasisymmetry(
         R_transform,
         Z_transform,
         L_transform,
-        p_transform,
-        i_transform,
+        p_profile,
+        i_profile,
     )
 
-    axis = i_transform.grid.axis
+    axis = i_profile.grid.axis
 
     # toroidal coordinate 2nd derivatives
     toroidal_coords["R_rtt"] = R_transform.transform(R_lmn, 1, 2, 0)
@@ -2247,17 +2253,17 @@ def compute_quasisymmetry(
     # B contravariant component derivatives
     magnetic_field["B0_tt"] = -(
         profiles["psi_r"]
-        / (2 * jnp.pi * jacobian["g"] ** 2)
+        / jacobian["g"] ** 2
         * (jacobian["g_tt"] - 2 * jacobian["g_t"] ** 2 / jacobian["g"])
     )
     magnetic_field["B0_zz"] = -(
         profiles["psi_r"]
-        / (2 * jnp.pi * jacobian["g"] ** 2)
+        / jacobian["g"] ** 2
         * (jacobian["g_zz"] - 2 * jacobian["g_z"] ** 2 / jacobian["g"])
     )
     magnetic_field["B0_tz"] = -(
         profiles["psi_r"]
-        / (2 * jnp.pi * jacobian["g"] ** 2)
+        / jacobian["g"] ** 2
         * (jacobian["g_tz"] - 2 * jacobian["g_t"] * jacobian["g_z"] / jacobian["g"])
     )
     magnetic_field["B^theta_tt"] = magnetic_field["B0_tt"] * (
@@ -2633,11 +2639,8 @@ def compute_quasisymmetry(
 
     quasisymmetry = {}
 
-    quasisymmetry["|grad(rho)|"] = jnp.sqrt(
-        dot(con_basis["e^rho"], con_basis["e^rho"], 0)
-    )
     quasisymmetry["|grad(psi)|"] = jnp.sqrt(
-        profiles["psi_r"] ** 2 * quasisymmetry["|grad(rho)|"] ** 2
+        profiles["psi_r"] ** 2 * dot(con_basis["e^rho"], con_basis["e^rho"], 0)
     )
 
     # B * grad(|B|) and derivatives
@@ -2658,48 +2661,11 @@ def compute_quasisymmetry(
         + magnetic_field["B^zeta"] * magnetic_field["|B|_zz"]
     )
 
-    # Triple Product QS metric (dimensionless)
-    quasisymmetry["QS_TP"] = (
-        magnetic_field["|B|_t"] * quasisymmetry["B*grad(|B|)_z"]
-        - magnetic_field["|B|_z"] * quasisymmetry["B*grad(|B|)_t"]
-    ) / (quasisymmetry["|grad(rho)|"] * magnetic_field["|B|"] ** 3)
-    quasisymmetry["QS_TP"] = put(quasisymmetry["QS_TP"], axis, 0)
-
-    # FIXME: singular points of QS Flux Function cause JAX issues
-    # Singular points (where B*grad(|B|)=0) are not known a-priori.
-    """
-    # Flux Function QS metric (T*m)
-    quasisymmetry["QS_FF"] = (
-        profiles["psi_r"]
-        / (jacobian["g"] * quasisymmetry["B*grad(|B|)"])
-        * (
-            +magnetic_field["B_zeta"] * magnetic_field["|B|_t"]
-            - magnetic_field["B_theta"] * magnetic_field["|B|_z"]
-        )
-    )
-    quasisymmetry["QS_FF_0"] = (
-        profiles["psi_r"]
-        * (
-            magnetic_field["B_zeta_t"] * magnetic_field["|B|_t"]
-            - magnetic_field["B_theta_t"] * magnetic_field["|B|_z"]
-            + magnetic_field["B_zeta"] * magnetic_field["|B|_tt"]
-            - magnetic_field["B_theta"] * magnetic_field["|B|_tz"]
-        )
-        / (
-            jacobian["g_t"] * quasisymmetry["B*grad(|B|)"]
-            + jacobian["g"] * quasisymmetry["B*grad(|B|)_t"]
-        )
-    )
-    singular = jnp.where(quasisymmetry["B*grad(|B|)"] == 0)[0]
-    quasisymmetry["QS_FF"] = put(
-        quasisymmetry["QS_FF"], singular, quasisymmetry["QS_FF_0"][singular]
-    )
-    """
-
     return (
         quasisymmetry,
         current_density,
         magnetic_field,
+        con_basis,
         jacobian,
         cov_basis,
         toroidal_coords,
