@@ -4,7 +4,7 @@ import warnings
 from collections.abc import MutableSequence
 from desc.backend import use_jax
 from desc.utils import Timer, isalmostequal
-from desc.configuration import _Configuration, format_boundary, format_profiles
+from desc.configuration import _Configuration
 from desc.io import IOAble
 from desc.boundary_conditions import get_boundary_condition, BoundaryCondition
 from desc.objective_funs import get_objective_function, ObjectiveFunction
@@ -12,6 +12,7 @@ from desc.optimize import Optimizer
 from desc.grid import Grid, LinearGrid, ConcentricGrid, QuadratureGrid
 from desc.transform import Transform
 from desc.perturbations import perturb, optimal_perturb
+from desc.geometry import FourierRZToroidalSurface, ZernikeRZToroidalSection
 
 
 class Equilibrium(_Configuration, IOAble):
@@ -64,7 +65,6 @@ class Equilibrium(_Configuration, IOAble):
         "_grid",
         "_node_pattern",
         "_transforms",
-        "_constraint",
         "_objective",
         "optimizer_results",
         "_optimizer",
@@ -80,17 +80,26 @@ class Equilibrium(_Configuration, IOAble):
         self._spectral_indexing = inputs.get("spectral_indexing", "fringe")
         self._node_pattern = inputs.get("node_pattern", "quad")
         self._solved = False
-        self._constraint = None
         self._objective = None
         self._optimizer = None
         self._set_grid()
         self._transforms = {}
         self._set_transforms()
-        self.constraint = inputs.get("bdry_mode", None)
         self.objective = inputs.get("objective", None)
         self.optimizer = inputs.get("optimizer", None)
         self.optimizer_results = {}
 
+    def __repr__(self):
+        """string form of the object"""
+        return (
+            type(self).__name__
+            + " at "
+            + str(hex(id(self)))
+            + " (L={}, M={}, N={}, NFP={}, sym={}, spectral_indexing={})".format(
+                self.L, self.M, self.N, self.NFP, self.sym, self.spectral_indexing
+            )
+        )
+
     @property
     def x0(self):
         """Return initial optimization vector before solution (ndarray)."""
@@ -204,31 +213,16 @@ class Equilibrium(_Configuration, IOAble):
             self._transforms["L"] = Transform(
                 self.grid, self.L_basis, derivs=0, build=False
             )
-            self._transforms["Rb"] = Transform(
-                self.grid, self.Rb_basis, derivs=0, build=False
-            )
-            self._transforms["Zb"] = Transform(
-                self.grid, self.Zb_basis, derivs=0, build=False
-            )
-            self._transforms["p"] = Transform(
-                self.grid, self.p_basis, derivs=1, build=False
-            )
-            self._transforms["i"] = Transform(
-                self.grid, self.i_basis, derivs=1, build=False
-            )
+            self.pressure.grid = self.grid
+            self.iota.grid = self.grid
 
         else:
             self.transforms["R"].change_resolution(self.grid, self.R_basis, build=False)
             self.transforms["Z"].change_resolution(self.grid, self.Z_basis, build=False)
             self.transforms["L"].change_resolution(self.grid, self.L_basis, build=False)
-            self.transforms["Rb"].change_resolution(
-                self.grid, self.Rb_basis, build=False
-            )
-            self.transforms["Zb"].change_resolution(
-                self.grid, self.Zb_basis, build=False
-            )
-            self.transforms["p"].change_resolution(self.grid, self.p_basis, build=False)
-            self.transforms["i"].change_resolution(self.grid, self.i_basis, build=False)
+        self.pressure.grid = self.grid
+        self.iota.grid = self.grid
+
         if self.objective is not None:
             derivs = self.objective.derivatives
             self.transforms["R"].change_derivatives(derivs, build=False)
@@ -323,7 +317,6 @@ class Equilibrium(_Configuration, IOAble):
             )
             and self.objective is not None
         ):
-            self.constraint = self.constraint.name
             self.objective = self.objective.name
 
     @property
@@ -357,43 +350,6 @@ class Equilibrium(_Configuration, IOAble):
     def solved(self, solved):
         self._solved = solved
 
-    @property
-    def constraint(self):
-        """Boundary condition currently assigned (BoundaryCondition)."""
-        if not hasattr(self, "_constraint"):
-            self._constraint = None
-        return self._constraint
-
-    @constraint.setter
-    def constraint(self, constraint):
-        if constraint is None:
-            self._constraint = constraint
-        elif isinstance(constraint, BoundaryCondition) and constraint.eq(
-            self.constraint
-        ):
-            return
-        elif isinstance(constraint, BoundaryCondition) and not constraint.eq(
-            self.constraint
-        ):
-            self._constraint = constraint
-        elif isinstance(constraint, str):
-            self._constraint = get_boundary_condition(
-                constraint,
-                R_basis=self.R_basis,
-                Z_basis=self.Z_basis,
-                L_basis=self.L_basis,
-                Rb_basis=self.Rb_basis,
-                Zb_basis=self.Zb_basis,
-                Rb_lmn=self.Rb_lmn,
-                Zb_lmn=self.Zb_lmn,
-                build=False,
-            )
-        else:
-            raise ValueError(
-                "objective should be of type 'BoundaryCondition' or string, "
-                + "got {}".format(constraint)
-            )
-
     @property
     def objective(self):
         """Objective function currently assigned (ObjectiveFunction)."""
@@ -413,17 +369,18 @@ class Equilibrium(_Configuration, IOAble):
             self._objective = objective
         elif isinstance(objective, str):
             self._set_transforms()
-            self._objective = get_objective_function(
+            objective = get_objective_function(
                 objective,
                 R_transform=self.transforms["R"],
                 Z_transform=self.transforms["Z"],
                 L_transform=self.transforms["L"],
-                Rb_transform=self.transforms["Rb"],
-                Zb_transform=self.transforms["Zb"],
-                p_transform=self.transforms["p"],
-                i_transform=self.transforms["i"],
-                BC_constraint=self.constraint,
+                p_profile=self.pressure,
+                i_profile=self.iota,
+                BC_constraint=self.surface.get_constraint(
+                    self.R_basis, self.Z_basis, self.L_basis
+                ),
             )
+            self.objective = objective
         else:
             raise ValueError(
                 "objective should be of type 'ObjectiveFunction' or string, "
@@ -460,15 +417,19 @@ class Equilibrium(_Configuration, IOAble):
     def initial(self):
         """Return initial equilibrium state from which it was solved (Equilibrium)."""
         p_modes = np.array(
-            [self.p_basis.modes[:, 0], self.p_l, np.zeros_like(self.p_l)]
+            [self.pressure.basis.modes[:, 0], self.p_l, np.zeros_like(self.p_l)]
         ).T
         i_modes = np.array(
-            [self.i_basis.modes[:, 0], np.zeros_like(self.i_l), self.i_l]
+            [self.iota.basis.modes[:, 0], np.zeros_like(self.i_l), self.i_l]
         ).T
         Rb_lmn = self.Rb_lmn.reshape((-1, 1))
         Zb_lmn = self.Zb_lmn.reshape((-1, 1))
-        Rb_modes = np.hstack([self.Rb_basis.modes, Rb_lmn, np.zeros_like(Rb_lmn)])
-        Zb_modes = np.hstack([self.Zb_basis.modes, np.zeros_like(Zb_lmn), Zb_lmn])
+        Rb_modes = np.hstack(
+            [self.surface.R_basis.modes, Rb_lmn, np.zeros_like(Rb_lmn)]
+        )
+        Zb_modes = np.hstack(
+            [self.surface.Z_basis.modes, np.zeros_like(Zb_lmn), Zb_lmn]
+        )
         inputs = {
             "sym": self.sym,
             "NFP": self.NFP,
@@ -562,6 +523,7 @@ class Equilibrium(_Configuration, IOAble):
             raise AttributeError(
                 "Equilibrium must have objective and optimizer defined before solving."
             )
+        self.objective = self.objective.name
 
         args = (self.Rb_lmn, self.Zb_lmn, self.p_l, self.i_l, self.Psi)
 
@@ -728,10 +690,37 @@ class EquilibriaFamily(IOAble, MutableSequence):
 
         """
         deltas = {}
-        Rb_lmn, Zb_lmn = format_boundary(
-            inputs["boundary"], equil.Rb_basis, equil.Zb_basis, equil.bdry_mode
-        )
-        p_l, i_l = format_profiles(inputs["profiles"], equil.p_basis, equil.i_basis)
+        if equil.bdry_mode == "lcfs":
+            s = FourierRZToroidalSurface(
+                inputs["boundary"][:, 3],
+                inputs["boundary"][:, 4],
+                inputs["boundary"][:, 1:3].astype(int),
+                inputs["boundary"][:, 1:3].astype(int),
+                equil.NFP,
+                equil.sym,
+            )
+            s.change_resolution(equil.M, equil.N)
+            Rb_lmn, Zb_lmn = s.R_mn, s.Z_mn
+        elif equil.bdry_mode == "poincare":
+            s = ZernikeRZToroidalSection(
+                inputs["boundary"][:, 3],
+                inputs["boundary"][:, 4],
+                inputs["boundary"][:, :2].astype(int),
+                inputs["boundary"][:, :2].astype(int),
+                equil.spectral_indexing,
+                equil.sym,
+            )
+            s.change_resolution(equil.L, equil.M)
+            Rb_lmn, Zb_lmn = s.R_lm, s.Z_lm
+
+        p_l = np.zeros_like(equil.pressure.params)
+        i_l = np.zeros_like(equil.iota.params)
+        for l, p, i in inputs["profiles"]:
+            idx_p = np.where(equil.pressure.basis.modes[:, 0] == int(l))[0]
+            idx_i = np.where(equil.iota.basis.modes[:, 0] == int(l))[0]
+            p_l[idx_p] = p
+            i_l[idx_i] = i
+
         if not np.allclose(Rb_lmn, equil.Rb_lmn):
             deltas["dRb"] = Rb_lmn - equil.Rb_lmn
         if not np.allclose(Zb_lmn, equil.Zb_lmn):
@@ -752,7 +741,7 @@ class EquilibriaFamily(IOAble, MutableSequence):
         print("Boundary ratio = {}".format(self.inputs[ii]["bdry_ratio"]))
         print("Pressure ratio = {}".format(self.inputs[ii]["pres_ratio"]))
         print("Perturbation Order = {}".format(self.inputs[ii]["pert_order"]))
-        print("Constraint: {}".format(equil.constraint.name))
+        print("Constraint: {}".format(equil.objective.BC_constraint.name))
         print("Objective: {}".format(equil.objective.name))
         print("Optimizer: {}".format(equil.optimizer.method))
         print("Function tolerance = {}".format(self.inputs[ii]["ftol"]))
@@ -851,32 +840,25 @@ class EquilibriaFamily(IOAble, MutableSequence):
                         "yellow",
                     )
                 )
+                if checkpoint_path is not None:
+                    if verbose > 0:
+                        print("Saving latest state")
+                    self.save(checkpoint_path)
                 break
 
-            # boundary condition
-            constraint = get_boundary_condition(
-                self.inputs[ii]["bdry_mode"],
-                R_basis=equil.R_basis,
-                Z_basis=equil.Z_basis,
-                L_basis=equil.L_basis,
-                Rb_basis=equil.Rb_basis,
-                Zb_basis=equil.Zb_basis,
-                Rb_lmn=equil.Rb_lmn,
-                Zb_lmn=equil.Zb_lmn,
-            )
-            equil.constraint = constraint
-
             # objective function
             objective = get_objective_function(
                 self.inputs[ii]["objective"],
                 R_transform=equil.transforms["R"],
                 Z_transform=equil.transforms["Z"],
                 L_transform=equil.transforms["L"],
-                Rb_transform=equil.transforms["Rb"],
-                Zb_transform=equil.transforms["Zb"],
-                p_transform=equil.transforms["p"],
-                i_transform=equil.transforms["i"],
-                BC_constraint=equil.constraint,
+                p_profile=equil.pressure,
+                i_profile=equil.iota,
+                BC_constraint=equil.surface.get_constraint(
+                    R_basis=equil.R_basis,
+                    Z_basis=equil.Z_basis,
+                    L_basis=equil.L_basis,
+                ),
                 use_jit=True,
             )
             # reuse old objective if possible to avoid recompiling
