@@ -51,6 +51,7 @@ class Grid(IOAble):
         if sort:
             self._sort_nodes()
         self._find_axis()
+        self._scale_weights()
 
     def _enforce_symmetry(self):
         """Enforces stellarator symmetry"""
@@ -70,6 +71,18 @@ class Grid(IOAble):
         """Finds indices of axis nodes"""
         self._axis = np.where(self.nodes[:, 0] == 0)[0]
 
+    def _scale_weights(self):
+        """Scales weights to sum to full volume and reduces weights for duplicated nodes"""
+
+        nodes = self.nodes.copy().astype(float)
+        nodes[:, 1] %= 2 * np.pi
+        nodes[:, 2] %= 2 * np.pi / self.NFP
+        _, inverse, counts = np.unique(
+            nodes, axis=0, return_inverse=True, return_counts=True
+        )
+        self._weights /= counts[inverse]
+        self._weights *= 4 * np.pi ** 2 / self._weights.sum()
+
     def _create_nodes(self, nodes):
         """Allows for custom node creation
 
@@ -85,7 +98,8 @@ class Grid(IOAble):
 
         """
         nodes = np.atleast_2d(nodes).reshape((-1, 3))
-        weights = np.ones(nodes.shape[0])
+        # make weights sum to 4pi^2
+        weights = np.ones(nodes.shape[0]) / nodes.shape[0] * 4 * np.pi ** 2
         self._L = len(np.unique(nodes[:, 0]))
         self._M = len(np.unique(nodes[:, 1]))
         self._N = len(np.unique(nodes[:, 2]))
@@ -249,6 +263,7 @@ class LinearGrid(Grid):
         self._enforce_symmetry()
         self._sort_nodes()
         self._find_axis()
+        self._scale_weights()
 
     def _create_nodes(
         self,
@@ -414,6 +429,7 @@ class QuadratureGrid(Grid):
         self._enforce_symmetry()  # symmetry is never enforced for Quadrature Grid
         self._sort_nodes()
         self._find_axis()
+        # quad grid should already be exact, so we don't scale weights
 
     def _create_nodes(self, L=1, M=1, N=1, NFP=1):
         """
@@ -553,6 +569,7 @@ class ConcentricGrid(Grid):
         self._enforce_symmetry()
         self._sort_nodes()
         self._find_axis()
+        self._scale_weights()
 
     def _create_nodes(self, L, M, N, NFP=1, axis=False, node_pattern="jacobi"):
         """
