@@ -32,13 +32,9 @@ class ObjectiveFunction(IOAble, ABC):
         transforms Z_lmn coefficients to real space
     L_transform : Transform
         transforms L_lmn coefficients to real space
-    Rb_transform : Transform
-        transforms Rb_lmn coefficients to real space
-    Zb_transform : Transform
-        transforms Zb_lmn coefficients to real space
-    p_transform : Transform
+    p_profile: Profile
         transforms p_l coefficients to real space
-    i_transform : Transform
+    i_profile: Profile
         transforms i_l coefficients to real space
     BC_constraint : BoundaryCondition
             linear constraint to enforce boundary conditions
@@ -51,10 +47,8 @@ class ObjectiveFunction(IOAble, ABC):
         "R_transform",
         "Z_transform",
         "L_transform",
-        "Rb_transform",
-        "Zb_transform",
-        "p_transform",
-        "i_transform",
+        "p_profile",
+        "i_profile",
         "BC_constraint",
         "use_jit",
     ]
@@ -66,10 +60,8 @@ class ObjectiveFunction(IOAble, ABC):
         R_transform,
         Z_transform,
         L_transform,
-        Rb_transform,
-        Zb_transform,
-        p_transform,
-        i_transform,
+        p_profile,
+        i_profile,
         BC_constraint,
         use_jit=True,
     ):
@@ -77,10 +69,8 @@ class ObjectiveFunction(IOAble, ABC):
         self.R_transform = R_transform
         self.Z_transform = Z_transform
         self.L_transform = L_transform
-        self.Rb_transform = Rb_transform
-        self.Zb_transform = Zb_transform
-        self.p_transform = p_transform
-        self.i_transform = i_transform
+        self.p_profile = p_profile
+        self.i_profile = i_profile
         self.BC_constraint = BC_constraint
         self.use_jit = use_jit
         self._set_up()
@@ -113,22 +103,6 @@ class ObjectiveFunction(IOAble, ABC):
             (self.derivatives[:, None] == self.L_transform.derivatives).all(-1).any(-1)
         ):
             self.L_transform.change_derivatives(self.derivatives, build=False)
-        if not all(
-            (self.derivatives[:, None] == self.Rb_transform.derivatives).all(-1).any(-1)
-        ):
-            self.Rb_transform.change_derivatives(self.derivatives, build=False)
-        if not all(
-            (self.derivatives[:, None] == self.Zb_transform.derivatives).all(-1).any(-1)
-        ):
-            self.Zb_transform.change_derivatives(self.derivatives, build=False)
-        if not all(
-            (self.derivatives[:, None] == self.p_transform.derivatives).all(-1).any(-1)
-        ):
-            self.p_transform.change_derivatives(self.derivatives, build=False)
-        if not all(
-            (self.derivatives[:, None] == self.i_transform.derivatives).all(-1).any(-1)
-        ):
-            self.i_transform.change_derivatives(self.derivatives, build=False)
 
     def set_derivatives(self, use_jit=True, block_size="auto"):
         """Set up derivatives of the objective function.
@@ -397,13 +371,9 @@ class ForceErrorGalerkin(ObjectiveFunction):
         transforms Z_lmn coefficients to real space
     L_transform : Transform
         transforms L_lmn coefficients to real space
-    Rb_transform : Transform
-        transforms Rb_lmn coefficients to real space
-    Zb_transform : Transform
-        transforms Zb_lmn coefficients to real space
-    p_transform : Transform
+    p_profile: Profile
         transforms p_l coefficients to real space
-    i_transform : Transform
+    i_profile: Profile
         transforms i_l coefficients to real space
     BC_constraint : BoundaryCondition
         linear constraint to enforce boundary conditions
@@ -417,10 +387,8 @@ class ForceErrorGalerkin(ObjectiveFunction):
         R_transform,
         Z_transform,
         L_transform,
-        Rb_transform,
-        Zb_transform,
-        p_transform,
-        i_transform,
+        p_profile,
+        i_profile,
         BC_constraint,
         use_jit=True,
     ):
@@ -429,10 +397,8 @@ class ForceErrorGalerkin(ObjectiveFunction):
             R_transform,
             Z_transform,
             L_transform,
-            Rb_transform,
-            Zb_transform,
-            p_transform,
-            i_transform,
+            p_profile,
+            i_profile,
             BC_constraint,
             use_jit,
         )
@@ -529,8 +495,8 @@ class ForceErrorGalerkin(ObjectiveFunction):
             self.R_transform,
             self.Z_transform,
             self.L_transform,
-            self.p_transform,
-            self.i_transform,
+            self.p_profile,
+            self.i_profile,
         )
 
         weights = self.R_transform.grid.weights
@@ -598,8 +564,8 @@ class ForceErrorGalerkin(ObjectiveFunction):
             self.R_transform,
             self.Z_transform,
             self.L_transform,
-            self.p_transform,
-            self.i_transform,
+            self.p_profile,
+            self.i_profile,
         )
 
         weights = self.R_transform.grid.weights
@@ -652,8 +618,8 @@ class ForceErrorGalerkin(ObjectiveFunction):
             self.R_transform,
             self.Z_transform,
             self.L_transform,
-            self.p_transform,
-            self.i_transform,
+            self.p_profile,
+            self.i_profile,
         )
 
         weights = self.R_transform.grid.weights
@@ -684,13 +650,9 @@ class ForceErrorNodes(ObjectiveFunction):
         transforms Z_lmn coefficients to real space
     L_transform : Transform
         transforms L_lmn coefficients to real space
-    Rb_transform : Transform
-        transforms Rb_lmn coefficients to real space
-    Zb_transform : Transform
-        transforms Zb_lmn coefficients to real space
-    p_transform : Transform
+    p_profile: Profile
         transforms p_l coefficients to real space
-    i_transform : Transform
+    i_profile: Profile
         transforms i_l coefficients to real space
     BC_constraint : BoundaryCondition
         linear constraint to enforce boundary conditions
@@ -782,8 +744,8 @@ class ForceErrorNodes(ObjectiveFunction):
             self.R_transform,
             self.Z_transform,
             self.L_transform,
-            self.p_transform,
-            self.i_transform,
+            self.p_profile,
+            self.i_profile,
         )
 
         weights = self.R_transform.grid.weights
@@ -871,8 +833,8 @@ class ForceErrorNodes(ObjectiveFunction):
             self.R_transform,
             self.Z_transform,
             self.L_transform,
-            self.p_transform,
-            self.i_transform,
+            self.p_profile,
+            self.i_profile,
         )
 
         weights = self.R_transform.grid.weights
@@ -909,13 +871,9 @@ class EnergyVolIntegral(ObjectiveFunction):
         transforms Z_lmn coefficients to real space
     L_transform : Transform
         transforms L_lmn coefficients to real space
-    Rb_transform : Transform
-        transforms Rb_lmn coefficients to real space
-    Zb_transform : Transform
-        transforms Zb_lmn coefficients to real space
-    p_transform : Transform
+    p_profile: Profile
         transforms p_l coefficients to real space
-    i_transform : Transform
+    i_profile: Profile
         transforms i_l coefficients to real space
     BC_constraint : BoundaryCondition
         linear constraint to enforce boundary conditions
@@ -929,10 +887,8 @@ class EnergyVolIntegral(ObjectiveFunction):
         R_transform,
         Z_transform,
         L_transform,
-        Rb_transform,
-        Zb_transform,
-        p_transform,
-        i_transform,
+        p_profile,
+        i_profile,
         BC_constraint,
         use_jit=True,
     ):
@@ -941,10 +897,8 @@ class EnergyVolIntegral(ObjectiveFunction):
             R_transform,
             Z_transform,
             L_transform,
-            Rb_transform,
-            Zb_transform,
-            p_transform,
-            i_transform,
+            p_profile,
+            i_profile,
             BC_constraint,
             use_jit,
         )
@@ -1024,8 +978,8 @@ class EnergyVolIntegral(ObjectiveFunction):
             self.R_transform,
             self.Z_transform,
             self.L_transform,
-            self.p_transform,
-            self.i_transform,
+            self.p_profile,
+            self.i_profile,
         )
 
         residual = energy["W"]
@@ -1103,8 +1057,8 @@ class EnergyVolIntegral(ObjectiveFunction):
             self.R_transform,
             self.Z_transform,
             self.L_transform,
-            self.p_transform,
-            self.i_transform,
+            self.p_profile,
+            self.i_profile,
         )
 
         print(
@@ -1127,13 +1081,9 @@ class QuasisymmetryTripleProduct(ObjectiveFunction):
         transforms Z_lmn coefficients to real space
     L_transform : Transform
         transforms L_lmn coefficients to real space
-    Rb_transform : Transform
-        transforms Rb_lmn coefficients to real space
-    Zb_transform : Transform
-        transforms Zb_lmn coefficients to real space
-    p_transform : Transform
+    p_profile: Profile
         transforms p_l coefficients to real space
-    i_transform : Transform
+    i_profile: Profile
         transforms i_l coefficients to real space
     BC_constraint : BoundaryCondition
         linear constraint to enforce boundary conditions
@@ -1214,6 +1164,7 @@ class QuasisymmetryTripleProduct(ObjectiveFunction):
             quasisymmetry,
             current_density,
             magnetic_field,
+            con_basis,
             jacobian,
             cov_basis,
             toroidal_coords,
@@ -1228,16 +1179,29 @@ class QuasisymmetryTripleProduct(ObjectiveFunction):
             self.R_transform,
             self.Z_transform,
             self.L_transform,
-            self.p_transform,
-            self.i_transform,
+            self.p_profile,
+            self.i_profile,
         )
 
-        return quasisymmetry["QS_TP"]
+        # QS triple product (T^4/m^2)
+        QS = (
+            profiles["psi_r"]
+            * (
+                magnetic_field["|B|_t"] * quasisymmetry["B*grad(|B|)_z"]
+                - magnetic_field["|B|_z"] * quasisymmetry["B*grad(|B|)_t"]
+            )
+            / jacobian["g"]
+        )
+
+        # normalization factor = <|B|>^4 / R^2
+        R0 = Rb_lmn[
+            jnp.where((self.Rb_transform.basis.modes == [0, 0, 0]).all(axis=1))[0]
+        ]
+        norm = jnp.mean(magnetic_field["|B|"] * jacobian["g"]) / jnp.mean(jacobian["g"])
+        return QS * R0 ** 2 / norm ** 4  # normalized QS error
 
     def compute_scalar(self, x, Rb_lmn, Zb_lmn, p_l, i_l, Psi):
-        """Compute the total quasisymetry error.
-
-        eg 1/2 sum(f**2)
+        """Compute the volume averaged quasi-symmetry error.
 
         Parameters
         ----------
@@ -1257,11 +1221,57 @@ class QuasisymmetryTripleProduct(ObjectiveFunction):
         Returns
         -------
         f : float
-            total quasisymmetry error
+            average quasi-symmetry error
+
         """
-        residual = self.compute(x, Rb_lmn, Zb_lmn, p_l, i_l, Psi)
-        residual = 1 / 2 * jnp.sum(residual ** 2)
-        return residual
+        if self.BC_constraint is not None and x.size == self.dimy:
+            # x is really 'y', need to recover full state vector
+            x = self.BC_constraint.recover_from_constraints(x, Rb_lmn, Zb_lmn)
+
+        R_lmn, Z_lmn, L_lmn = unpack_state(
+            x, self.R_transform.basis.num_modes, self.Z_transform.basis.num_modes
+        )
+
+        (
+            quasisymmetry,
+            current_density,
+            magnetic_field,
+            con_basis,
+            jacobian,
+            cov_basis,
+            toroidal_coords,
+            profiles,
+        ) = compute_quasisymmetry(
+            Psi,
+            R_lmn,
+            Z_lmn,
+            L_lmn,
+            p_l,
+            i_l,
+            self.R_transform,
+            self.Z_transform,
+            self.L_transform,
+            self.p_profile,
+            self.i_profile,
+        )
+
+        # QS triple product (T^4/m^2)
+        QS = (
+            profiles["psi_r"]
+            * (
+                magnetic_field["|B|_t"] * quasisymmetry["B*grad(|B|)_z"]
+                - magnetic_field["|B|_z"] * quasisymmetry["B*grad(|B|)_t"]
+            )
+            / jacobian["g"]
+        )
+
+        # normalization factor = <|B|>^4 / R^2
+        R0 = Rb_lmn[
+            jnp.where((self.Rb_transform.basis.modes == [0, 0, 0]).all(axis=1))[0]
+        ]
+        norm = jnp.mean(magnetic_field["|B|"] * jacobian["g"]) / jnp.mean(jacobian["g"])
+        f = QS * R0 ** 2 / norm ** 4  # normalized QS error
+        return jnp.mean(jnp.abs(f) * jacobian["g"]) / jnp.mean(jacobian["g"])
 
     def callback(self, x, Rb_lmn, Zb_lmn, p_l, i_l, Psi):
         """Print the rms errors for quasisymmetry.
@@ -1300,13 +1310,9 @@ class QuasisymmetryFluxFunction(ObjectiveFunction):
         transforms Z_lmn coefficients to real space
     L_transform : Transform
         transforms L_lmn coefficients to real space
-    Rb_transform : Transform
-        transforms Rb_lmn coefficients to real space
-    Zb_transform : Transform
-        transforms Zb_lmn coefficients to real space
-    p_transform : Transform
+    p_profile: Profile
         transforms p_l coefficients to real space
-    i_transform : Transform
+    i_profile: Profile
         transforms i_l coefficients to real space
     BC_constraint : BoundaryCondition
         linear constraint to enforce boundary conditions
@@ -1320,10 +1326,8 @@ class QuasisymmetryFluxFunction(ObjectiveFunction):
         R_transform,
         Z_transform,
         L_transform,
-        Rb_transform,
-        Zb_transform,
-        p_transform,
-        i_transform,
+        p_profile,
+        i_profile,
         BC_constraint,
         use_jit=True,
     ):
@@ -1332,10 +1336,8 @@ class QuasisymmetryFluxFunction(ObjectiveFunction):
             R_transform,
             Z_transform,
             L_transform,
-            Rb_transform,
-            Zb_transform,
-            p_transform,
-            i_transform,
+            p_profile,
+            i_profile,
             BC_constraint,
             use_jit,
         )
@@ -1422,6 +1424,7 @@ class QuasisymmetryFluxFunction(ObjectiveFunction):
             quasisymmetry,
             current_density,
             magnetic_field,
+            con_basis,
             jacobian,
             cov_basis,
             toroidal_coords,
@@ -1436,18 +1439,41 @@ class QuasisymmetryFluxFunction(ObjectiveFunction):
             self.R_transform,
             self.Z_transform,
             self.L_transform,
-            self.p_transform,
-            self.i_transform,
+            self.p_profile,
+            self.i_profile,
         )
 
-        # normalize to flux surface average
-        QS_FF = quasisymmetry["QS_FF"] / jnp.mean(quasisymmetry["QS_FF"]) - 1
-        return QS_FF
+        # M/N (type of QS)
+        helicity = 1.0 / 1.0
+
+        # covariant Boozer components
+        G = jnp.mean(magnetic_field["B_zeta"] * jacobian["g"]) / jnp.mean(
+            jacobian["g"]
+        )  # poloidal current
+        I = jnp.mean(magnetic_field["B_theta"] * jacobian["g"]) / jnp.mean(
+            jacobian["g"]
+        )  # toroidal current
+
+        # flux function C=C(rho)
+        C = (helicity * G + I) / (helicity * profiles["iota"] - 1)
+
+        # QS flux function (T^3)
+        QS = (
+            profiles["psi_r"]
+            / jacobian["g"]
+            * (
+                magnetic_field["B_zeta"] * magnetic_field["|B|_t"]
+                - magnetic_field["B_theta"] * magnetic_field["|B|_z"]
+            )
+            - C * quasisymmetry["B*grad(|B|)"]
+        )
+
+        # normalization factor = <|B|>^3
+        norm = jnp.mean(magnetic_field["|B|"] * jacobian["g"]) / jnp.mean(jacobian["g"])
+        return QS / norm ** 3  # normalized QS error
 
     def compute_scalar(self, x, Rb_lmn, Zb_lmn, p_l, i_l, Psi):
-        """Compute the total quasisymetry error.
-
-        eg 1/2 sum(f**2)
+        """Compute the volume averaged quasi-symmetry error.
 
         Parameters
         ----------
@@ -1467,11 +1493,67 @@ class QuasisymmetryFluxFunction(ObjectiveFunction):
         Returns
         -------
         f : float
-            total quasisymmetry error
+            average quasi-symmetry error
+
         """
-        residual = self.compute(x, Rb_lmn, Zb_lmn, p_l, i_l, Psi)
-        residual = 1 / 2 * jnp.sum(residual ** 2)
-        return residual
+        if self.BC_constraint is not None and x.size == self.dimy:
+            # x is really 'y', need to recover full state vector
+            x = self.BC_constraint.recover_from_constraints(x, Rb_lmn, Zb_lmn)
+
+        R_lmn, Z_lmn, L_lmn = unpack_state(
+            x, self.R_transform.basis.num_modes, self.Z_transform.basis.num_modes
+        )
+
+        (
+            quasisymmetry,
+            current_density,
+            magnetic_field,
+            con_basis,
+            jacobian,
+            cov_basis,
+            toroidal_coords,
+            profiles,
+        ) = compute_quasisymmetry(
+            Psi,
+            R_lmn,
+            Z_lmn,
+            L_lmn,
+            p_l,
+            i_l,
+            self.R_transform,
+            self.Z_transform,
+            self.L_transform,
+            self.p_profile,
+            self.i_profile,
+        )
+
+        # covariant Boozer components
+        G = jnp.mean(magnetic_field["B_zeta"] * jacobian["g"]) / jnp.mean(
+            jacobian["g"]
+        )  # poloidal current
+        I = jnp.mean(magnetic_field["B_theta"] * jacobian["g"]) / jnp.mean(
+            jacobian["g"]
+        )  # toroidal current
+
+        helicity = 1.0 / 1.0  # M/N (type of QS)
+        # flux function C=C(rho)
+        C = (helicity * G + I) / (helicity * profiles["iota"] - 1)
+
+        # QS flux function (T^3)
+        QS = (
+            profiles["psi_r"]
+            / jacobian["g"]
+            * (
+                magnetic_field["B_zeta"] * magnetic_field["|B|_t"]
+                - magnetic_field["B_theta"] * magnetic_field["|B|_z"]
+            )
+            - C * quasisymmetry["B*grad(|B|)"]
+        )
+
+        # normalization factor = <|B|>^3
+        norm = jnp.mean(magnetic_field["|B|"] * jacobian["g"]) / jnp.mean(jacobian["g"])
+        f = QS / norm ** 3  # normalized QS error
+        return jnp.mean(jnp.abs(f) * jacobian["g"]) / jnp.mean(jacobian["g"])
 
     def callback(self, x, Rb_lmn, Zb_lmn, p_l, i_l, Psi):
         """Print the rms errors for quasisymmetry.
@@ -1504,10 +1586,8 @@ def get_objective_function(
     R_transform,
     Z_transform,
     L_transform,
-    Rb_transform,
-    Zb_transform,
-    p_transform,
-    i_transform,
+    p_profile,
+    i_profile,
     BC_constraint=None,
     use_jit=True,
 ):
@@ -1523,13 +1603,9 @@ def get_objective_function(
         transforms Z_lmn coefficients to real space
     L_transform : Transform
         transforms L_lmn coefficients to real space
-    Rb_transform : Transform
-        transforms Rb_lmn coefficients to real space
-    Zb_transform : Transform
-        transforms Zb_lmn coefficients to real space
-    p_transform : Transform
+    p_profile: Profile
         transforms p_l coefficients to real space
-    i_transform : Transform
+    i_profile: Profile
         transforms i_l coefficients to real space
     BC_constraint : BoundaryCondition
         linear constraint to enforce boundary conditions
@@ -1547,10 +1623,8 @@ def get_objective_function(
             R_transform=R_transform,
             Z_transform=Z_transform,
             L_transform=L_transform,
-            Rb_transform=Rb_transform,
-            Zb_transform=Zb_transform,
-            p_transform=p_transform,
-            i_transform=i_transform,
+            p_profile=p_profile,
+            i_profile=i_profile,
             BC_constraint=BC_constraint,
             use_jit=use_jit,
         )
@@ -1559,10 +1633,8 @@ def get_objective_function(
             R_transform=R_transform,
             Z_transform=Z_transform,
             L_transform=L_transform,
-            Rb_transform=Rb_transform,
-            Zb_transform=Zb_transform,
-            p_transform=p_transform,
-            i_transform=i_transform,
+            p_profile=p_profile,
+            i_profile=i_profile,
             BC_constraint=BC_constraint,
             use_jit=use_jit,
         )
@@ -1571,10 +1643,8 @@ def get_objective_function(
             R_transform=R_transform,
             Z_transform=Z_transform,
             L_transform=L_transform,
-            Rb_transform=Rb_transform,
-            Zb_transform=Zb_transform,
-            p_transform=p_transform,
-            i_transform=i_transform,
+            p_profile=p_profile,
+            i_profile=i_profile,
             BC_constraint=BC_constraint,
             use_jit=use_jit,
         )
@@ -1589,33 +1659,3 @@ def get_objective_function(
         )
 
     return obj_fun
-
-
-"""
-QS derivatives (just here so we don't loose them)
-            derivatives = np.array(
-                [
-                    [0, 0, 0],
-                    [1, 0, 0],
-                    [0, 1, 0],
-                    [0, 0, 1],
-                    [2, 0, 0],
-                    [1, 1, 0],
-                    [1, 0, 1],
-                    [0, 2, 0],
-                    [0, 1, 1],
-                    [0, 0, 2],
-                    [3, 0, 0],
-                    [2, 1, 0],
-                    [2, 0, 1],
-                    [1, 2, 0],
-                    [1, 1, 1],
-                    [1, 0, 2],
-                    [0, 3, 0],
-                    [0, 2, 1],
-                    [0, 1, 2],
-                    [0, 0, 3],
-                    [2, 2, 0],
-                ]
-            )
-"""
