@@ -4,6 +4,8 @@ import numpy as np
 import re
 from termcolor import colored
 import warnings
+from scipy.interpolate import Rbf
+from scipy.integrate import solve_ivp
 
 from desc.grid import Grid, LinearGrid
 from desc.basis import jacobi, fourier
@@ -186,13 +188,19 @@ def _get_plot_axes(grid):
     return tuple(plot_axes)
 
 
-def plot_coefficients(eq, ax=None):
-    """Plot 1D profiles.
+def plot_coefficients(eq, L=True, M=True, N=True, ax=None):
+    """Plot spectral coefficient magnitudes vs spectral mode number.
 
     Parameters
     ----------
     eq : Equilibrium
         object from which to plot
+    L : bool
+        wheter to include radial mode numbers in the x-axis or not (Default = True)
+    M : bool
+        wheter to include poloidal mode numbers in the x-axis or not (Default = True)
+    N : bool
+        wheter to include toroidal mode numbers in the x-axis or not (Default = True)
     ax : matplotlib AxesSubplot, optional
         axis to plot on
 
@@ -204,15 +212,37 @@ def plot_coefficients(eq, ax=None):
         axes being plotted to
 
     """
+    lmn = np.array([], dtype=int)
+    xlabel = ""
+    if L:
+        lmn = np.append(lmn, np.array([0]))
+        xlabel += "l"
+        if M or N:
+            xlabel += " + "
+    if M:
+        lmn = np.append(lmn, np.array([1]))
+        xlabel += "|m|"
+        if N:
+            xlabel += " + "
+    if N:
+        lmn = np.append(lmn, np.array([2]))
+        xlabel += "|n|"
+
     fig, ax = _format_ax(ax, rows=1, cols=3)
 
-    ax[0, 0].semilogy(np.sum(np.abs(eq.R_basis.modes), axis=1), np.abs(eq.R_lmn), "bo")
-    ax[0, 1].semilogy(np.sum(np.abs(eq.Z_basis.modes), axis=1), np.abs(eq.Z_lmn), "bo")
-    ax[0, 2].semilogy(np.sum(np.abs(eq.L_basis.modes), axis=1), np.abs(eq.L_lmn), "bo")
+    ax[0, 0].semilogy(
+        np.sum(np.abs(eq.R_basis.modes[:, lmn]), axis=1), np.abs(eq.R_lmn), "bo"
+    )
+    ax[0, 1].semilogy(
+        np.sum(np.abs(eq.Z_basis.modes[:, lmn]), axis=1), np.abs(eq.Z_lmn), "bo"
+    )
+    ax[0, 2].semilogy(
+        np.sum(np.abs(eq.L_basis.modes[:, lmn]), axis=1), np.abs(eq.L_lmn), "bo"
+    )
 
-    ax[0, 0].set_xlabel("l + |m| + |n|")
-    ax[0, 1].set_xlabel("l + |m| + |n|")
-    ax[0, 2].set_xlabel("l + |m| + |n|")
+    ax[0, 0].set_xlabel(xlabel)
+    ax[0, 1].set_xlabel(xlabel)
+    ax[0, 2].set_xlabel(xlabel)
 
     ax[0, 0].set_title("$|R_{lmn}|$")
     ax[0, 1].set_title("$|Z_{lmn}|$")
@@ -342,7 +372,7 @@ def plot_2d(eq, name, grid=None, ax=None, log=False, norm_F=False, **kwargs):
         if norm_F:
             contourf_kwargs["levels"] = kwargs.get("levels", np.logspace(-6, 0, 7))
         else:
-            logmin = np.floor(np.nanmin(np.log10(data))).astype(int)
+            logmin = max(np.floor(np.nanmin(np.log10(data))).astype(int), -16)
             logmax = np.ceil(np.nanmax(np.log10(data))).astype(int)
             contourf_kwargs["levels"] = kwargs.get(
                 "levels", np.logspace(logmin, logmax, logmax - logmin + 1)
@@ -410,7 +440,7 @@ def plot_3d(eq, name, grid=None, ax=None, log=False, all_field_periods=True, **k
     """
     nfp = 1 if all_field_periods else eq.NFP
     if grid is None:
-        grid_kwargs = {"M": 46, "N": 46, "NFP": nfp}
+        grid_kwargs = {"M": 32, "N": int(32 * eq.NFP), "NFP": nfp}
         grid = _get_grid(**grid_kwargs)
     plot_axes = _get_plot_axes(grid)
     if len(plot_axes) != 2:
@@ -452,7 +482,8 @@ def plot_3d(eq, name, grid=None, ax=None, log=False, all_field_periods=True, **k
         norm = matplotlib.colors.Normalize(vmin=minn, vmax=maxx)
     m = plt.cm.ScalarMappable(cmap=plt.cm.jet, norm=norm)
     m.set_array([])
-
+    alpha = kwargs.get('alpha',1)
+    
     ax.plot_surface(
         X,
         Y,
@@ -463,6 +494,7 @@ def plot_3d(eq, name, grid=None, ax=None, log=False, all_field_periods=True, **k
         vmax=maxx,
         rstride=1,
         cstride=1,
+        alpha=alpha,
     )
     fig.colorbar(m)
 
@@ -691,31 +723,27 @@ def plot_surfaces(eq, r_grid=None, t_grid=None, ax=None, **kwargs):
         if zeta.size != t_zeta.size or not np.allclose(zeta, t_zeta):
             raise ValueError(
                 colored(
-                    "r_grid and t_grid should have the same zeta planes, got r_grid={}, t_grid{}".format(
-                        zeta, t_zeta
-                    ),
+                    "r_grid and t_grid should have the same zeta planes, "
+                    + "got r_grid={}, t_grid{}".format(zeta, t_zeta),
                     "red",
                 )
             )
+
+    # Note: theta* (also known as vartheta) is the poloidal straight field-line anlge in
+    # PEST-like flux coordinates
+
+    v_grid = Grid(eq.compute_theta_coords(t_grid.nodes))
     rows = np.floor(np.sqrt(nzeta)).astype(int)
     cols = np.ceil(nzeta / rows).astype(int)
 
     r_coords = eq.compute_toroidal_coords(r_grid)
-    t_coords = eq.compute_toroidal_coords(t_grid)
-
-    # theta coordinates cooresponding to linearly spaced vartheta angles
-    v_nodes = t_grid.nodes
-    v_nodes[:, 1] = t_grid.nodes[:, 1] - t_coords["lambda"]
-    v_grid = Grid(v_nodes)
-    with warnings.catch_warnings():
-        warnings.simplefilter("ignore")
-        v_coords = eq.compute_toroidal_coords(v_grid)
+    v_coords = eq.compute_toroidal_coords(v_grid)
 
     # rho contours
     Rr = r_coords["R"].reshape((r_grid.M, r_grid.L, r_grid.N), order="F")
     Zr = r_coords["Z"].reshape((r_grid.M, r_grid.L, r_grid.N), order="F")
 
-    # theta contours
+    # vartheta contours
     Rv = v_coords["R"].reshape((t_grid.M, t_grid.L, t_grid.N), order="F")
     Zv = v_coords["Z"].reshape((t_grid.M, t_grid.L, t_grid.N), order="F")
 
@@ -812,7 +840,7 @@ def _compute(eq, name, grid):
             out = eq.compute_magnetic_pressure_gradient(grid)[_name_key(name_dict)]
         elif name_dict["base"] in ["F", "|F|", "|grad(p)|", "|grad(rho)|", "|beta|"]:
             out = eq.compute_force_error(grid)[_name_key(name_dict)]
-        elif name_dict["base"] in ["QS", "B*grad(|B|)"]:
+        elif name_dict["base"] in ["|grad(psi)|", "B*grad(|B|)"]:
             out = eq.compute_quasisymmetry(grid)[_name_key(name_dict)]
         else:
             raise NotImplementedError(
@@ -907,7 +935,7 @@ def _format_name(name):
         "|grad(p)|": r"(\mathrm{N}/\mathrm{m}^3)",
         "|grad(rho)|": r"(\mathrm{m}^{-1})",
         "|beta|": r"(\mathrm{m}^{-1})",
-        "QS": "",
+        "|grad(psi)|": r"(\mathrm{T} \cdot \mathrm{m})",
         "B*grad(|B|)": r"(\mathrm{T}^2/\mathrm{m})",
     }
     name_dict["units"] = units[name_dict["base"]]
@@ -1488,3 +1516,231 @@ def plot_logo(savepath=None, **kwargs):
         fig.savefig(savepath, facecolor=fig.get_facecolor(), edgecolor="none")
 
     return fig, ax
+
+def plot_field_lines(eq, rho, seed_thetas=0, phi_end=2*np.pi, grid=None, ax=None, B_interp=None, return_B_interp = False, **kwargs):
+    """Traces field lines on specified flux surface at specified initial theta seed locations, then plots them. 
+    Field lines integrated by first fitting the magnetic field with radial basis functions (RBF) in R,Z,phi, then integrating the field line
+    from phi=0 up to the specified phi angle, by solving:
+        
+    :math:`\\frac{dR}{d\phi} = \\frac{RB_R}{B_{\phi}} , \\frac{dZ}{d\phi} = \\frac{RB_Z}{B_{\phi}}`
+    
+    :math:`B_R = \mathbf{B} \cdot \hat{\mathbf{R}} = (B^{\\theta} \mathbf{e}_{\\theta} + B^{\zeta} \mathbf{e}_{\zeta}) \cdot \hat{\mathbf{R}} = B^{\\theta} \\frac{\partial R}{\partial \\theta} + B^{\zeta} \\frac{\partial R}{\partial \zeta}`
+
+    :math:`B_Z = \mathbf{B} \cdot \hat{\mathbf{Z}} = (B^{\\theta} \mathbf{e}_{\\theta} + B^{\zeta} \mathbf{e}_{\zeta}) \cdot \hat{\mathbf{Z}} = B^{\\theta} \\frac{\partial Z}{\partial \\theta} + B^{\zeta} \\frac{\partial Z}{\partial \zeta}`
+
+    :math:`B_{\phi} = \mathbf{B} \cdot \hat{\mathbf{\phi}} = R B^{\zeta}`
+
+    Parameters
+    ----------
+    eq : Equilibrium
+        object from which to plot
+    name : str
+        name of variable to plot
+    rho : float
+        flux surface to trace field lines at
+    seed_thetas : float or array-like of floats
+        theta positions at which to seed magnetic field lines, if array-like, will plot multiple field lines
+    phi_end: float
+        phi to integrate field line until, in radians. Default is 2*pi
+    grid : Grid, optional
+        grid of rho, theta, zeta coordinates used to evaluate magnetic field at, which is then interpolated with RBF
+    ax : matplotlib AxesSubplot, optional
+        axis to plot on
+    B_interp : dict of scipy.interpolate.rbf.Rbf or equivalent call signature interplators, optional
+        if not None, uses the passed-in interpolation objects instead of fitting the magnetic field with Rbf's. Useful
+        if have already ran plot_field_lines once and want to change the seed thetas or how far to integrate in phi.
+        Dict should have the following keys: ['B_R'], ['B_Z'], and ['B_phi'], corresponding to the interplating object for 
+        each cylindrical component of the magnetic field.
+    return_B_interp: bool, default False
+        If true, in addition to returning the fig, axis and field line coordinates, will also return the dictionary of interpolating radial basis functions
+        interpolating the magnetic field in (R,phi,Z)
+
+
+    Returns
+    -------
+    fig : matplotlib.figure.Figure
+        figure being plotted to
+    ax : matplotlib.axes.Axes or ndarray of Axes
+        axes being plotted to
+    field_line_coords : dict
+        dict containing the R,phi,Z coordinates of each field line traced. Dictionary entries are lists
+        corresponding to the field lines for each seed_theta given. Also contains the scipy IVP solutions for info
+        on how each line was integarted
+    B_interp : dict, only returned if return_B_interp is True
+        dict of scipy.interpolate.rbf.Rbf or equivalent call signature interplators, which interpolate the cylindrical
+        components of magnetic field in (R,phi,Z)
+        Dict has the following keys: ['B_R'], ['B_Z'], and ['B_phi'], corresponding to the interplating object for 
+        each cylindrical component of the magnetic field, and the interpolators have call signature
+        B(R,phi,Z) = interpolator(R,phi,Z)
+
+    """
+    nfp = 1
+    if grid is None:
+        grid_kwargs = {"M": 30, "N": 30 , "L": 20, "NFP": nfp, "axis" : False}
+        grid = _get_grid(**grid_kwargs)
+        
+    fig, ax = _format_ax(ax, is3d=True, figsize=kwargs.get("figsize", None))
+    
+    # check how many field lines to plot
+    if seed_thetas is list:
+        n_lines = len(seed_thetas)
+    elif isinstance(seed_thetas,np.ndarray):
+        n_lines = seed_thetas.size
+    else:
+        n_lines = 1
+    phi0 = kwargs.get('phi0',0)
+    # calculate R,phi,Z of nodes in grid
+    toroidal_coords = eq.compute_toroidal_coords(grid=grid)
+    # calculate cylindrical B
+    magnetic_field = eq.compute_magnetic_field(grid=grid)
+
+    phis = grid.nodes[:,2]
+    BR = magnetic_field['B_R']
+    BZ = magnetic_field['B_Z']
+    Bphi = magnetic_field['B_phi']
+    Rs = toroidal_coords['R']
+    Zs = toroidal_coords['Z']
+    if B_interp is None: # must fit RBfs to interpolate B field in R,phi,Z
+        print('Fitting magnetic field with radial basis functions in R,phi,Z (may take a few minutes)')
+        BRi = Rbf(Rs,Zs,phis,BR)
+        BZi = Rbf(Rs,Zs,phis,BZ)
+        Bphii = Rbf(Rs,Zs,phis,Bphi)
+        B_interp = {"B_R": BRi, "B_Z": BZi, "B_phi": Bphii}
+
+    
+    
+    field_line_coords = {"Rs" : [], "Zs" : [], "phis" : [], "IVP solutions": [], "seed_thetas": seed_thetas}
+    if n_lines > 1:
+        for theta in seed_thetas:
+            field_line_Rs,field_line_phis,field_line_Zs, sol = _field_line_Rbf(rho,theta,phi_end,grid,toroidal_coords,B_interp,phi0)
+            field_line_coords["Rs"].append(field_line_Rs)
+            field_line_coords["Zs"].append(field_line_Zs)
+            field_line_coords["phis"].append(field_line_phis)
+            field_line_coords["IVP solutions"].append(sol)
+            
+    else:
+        field_line_Rs,field_line_phis,field_line_Zs,sol = _field_line_Rbf(rho,seed_thetas,phi_end,grid,toroidal_coords,B_interp,phi0)
+        field_line_coords["Rs"].append(field_line_Rs)
+        field_line_coords["Zs"].append(field_line_Zs)
+        field_line_coords["phis"].append(field_line_phis)
+        field_line_coords["IVP solutions"].append(sol)
+    
+    for i,solution in enumerate(field_line_coords["IVP solutions"]):
+        if not solution.success:
+            print('Integration from seed theta %1.2f radians was not successful!' %seed_thetas[i])
+
+    for i in range(n_lines):
+        xline = np.asarray(field_line_coords["Rs"][i])*np.cos(field_line_coords["phis"][i])
+        yline = np.asarray(field_line_coords["Rs"][i])*np.sin(field_line_coords["phis"][i])
+    
+        ax.plot(xline, yline, field_line_coords["Zs"][i],linewidth=2)
+
+    ax.set_xlabel(_axis_labels_XYZ[0])
+    ax.set_ylabel(_axis_labels_XYZ[1])
+    ax.set_zlabel(_axis_labels_XYZ[2])
+    ax.set_title('%d Magnetic Field Lines Traced On $\\rho=%1.2f$ Surface' %(n_lines,rho))
+    fig.set_tight_layout(True)
+
+    # need this stuff to make all the axes equal, ax.axis('equal') doesnt work for 3d
+    x_limits = ax.get_xlim3d()
+    y_limits = ax.get_ylim3d()
+    z_limits = ax.get_zlim3d()
+
+    x_range = abs(x_limits[1] - x_limits[0])
+    x_middle = np.mean(x_limits)
+    y_range = abs(y_limits[1] - y_limits[0])
+    y_middle = np.mean(y_limits)
+    z_range = abs(z_limits[1] - z_limits[0])
+    z_middle = np.mean(z_limits)
+
+    # The plot bounding box is a sphere in the sense of the infinity
+    # norm, hence I call half the max range the plot radius.
+    plot_radius = 0.5 * max([x_range, y_range, z_range])
+
+    ax.set_xlim3d([x_middle - plot_radius, x_middle + plot_radius])
+    ax.set_ylim3d([y_middle - plot_radius, y_middle + plot_radius])
+    ax.set_zlim3d([z_middle - plot_radius, z_middle + plot_radius])
+    
+    if return_B_interp:
+        return fig, ax, field_line_coords, B_interp
+    else:
+        return fig, ax, field_line_coords
+
+def _find_idx(rho0,theta0,phi0,grid):
+    '''
+    Finds the node index corresponding to the rho,theta,zeta node closest to the given rho0,theta0,phi0
+
+    Parameters
+    ----------
+    rho0 : float
+        rho to find closest grid point to.
+    theta0 : float
+        theta to find closest grid point to.
+    phi0 : float
+        phi to find closest grid point to.
+    grid : Grid
+        grid to find closest point on
+
+
+    Returns
+    -------
+    idx_pt : int
+        index of the grid node closest to the given point.
+
+    '''
+    rhos = grid.nodes[:,0]
+    thetas = grid.nodes[:,1]
+    phis = grid.nodes[:,2]
+    
+    if theta0<0:
+        theta0 = (2*np.pi+theta0)
+    if theta0 > 2*np.pi:
+        theta0 == np.mod(theta0 ,2*np.pi)
+    if phi0<0:
+        phi0 = (2*np.pi+phi0)
+    if phi0 > 2*np.pi:
+        phi0 == np.mod(phi0,2*np.pi)
+        
+    bool1 = np.logical_and(np.abs(rhos-rho0)==np.min(np.abs(rhos-rho0)), np.abs(thetas-theta0)==np.min(np.abs(thetas-theta0))) 
+    bool2 = np.logical_and(bool1,np.abs(phis-phi0)==np.min(np.abs(phis-phi0)))
+    idx_pt = np.where(bool2==True)[0][0]
+    return idx_pt
+
+
+def _field_line_Rbf(rho,theta0,phi_end,grid,toroidal_coords,B_interp,phi0=0):
+    ''' Takes the initial poloidal angle you want to seed a field line at (at phi=0),
+        and integrates along the field line to the specified phi_end. returns fR,fZ,fPhi, the R,Z,Phi coordinates of the field line trajectory'''
+    Rs = toroidal_coords['R']
+    Zs = toroidal_coords['Z']
+    
+    
+    fR=[]
+    fZ=[]
+    fPhi=[]
+    idx0 = _find_idx(rho,theta0,phi0,grid)
+    curr_R = Rs[idx0]
+    curr_Z = Zs[idx0]
+    fR.append(curr_R)
+    fZ.append(curr_Z)
+    fPhi.append(phi0)
+
+
+    # integrate field lines in Phi
+    print('Integrating Magnetic Field Line Equation from seed theta = %f radians' %theta0)
+    y0 = [fR[0],fZ[0]]
+    def rhs(phi,y):
+        """ RHS of magnetic field line eqn"""
+        dRdphi = y[0]*B_interp['B_R'](y[0],y[1],np.mod(phi,2*np.pi)) / B_interp['B_phi'](y[0],y[1],np.mod(phi,2*np.pi))
+        dZdphi = y[0]*B_interp['B_Z'](y[0],y[1],np.mod(phi,2*np.pi)) / B_interp['B_phi'](y[0],y[1],np.mod(phi,2*np.pi))
+        return [dRdphi,dZdphi]
+    n_tries=1
+    max_step=0.01
+    sol = solve_ivp(rhs,[0,phi_end],y0,max_step=max_step)
+    while not sol.success and n_tries<4:
+        max_step = 0.5*max_step
+        n_tries += 1
+        sol = solve_ivp(rhs,[0,phi_end],y0,max_step=max_step)
+    fR = sol.y[0,:]
+    fZ = sol.y[1,:]
+    fPhi = sol.t
+    return fR,fPhi,fZ,sol
\ No newline at end of file
